@@ -1,4 +1,6 @@
+import os
 from datetime import datetime
+
 from fabric.api import *
 from fabric.colors import *
 from fabric.contrib.files import *
@@ -6,7 +8,6 @@ from fabric.contrib.console import confirm
 from fabric.operations import require
 from fabric.operations import open_shell
 from fabric.tasks import Task
-import os
 
 import deploytool.utils as utils
 
@@ -21,7 +22,7 @@ class RemoteHost(Task):
         'hosts',
         'project_name',
         'project_name_prefix',
-        'projects_root',
+        'vhosts_path',
     ]
 
     def __init__(self, *args, **kwargs):
@@ -40,23 +41,22 @@ class RemoteHost(Task):
         # check if all required project settings are present in fabric environment
         [require(r) for r in self.requirements]
 
-        project_name = '%s%s' % (env.project_name_prefix, env.project_name)
-        project_path = os.path.join(env.projects_root, project_name)
+        vhost_folder = '%s%s' % (env.project_name_prefix, env.project_name)
+        vhost_path = os.path.join(env.vhosts_path, vhost_folder)
 
         print(green('\nInitializing fabric environment for %s.' % magenta(self.name)))
         env.update({
-            'cache_path': os.path.join(project_path, 'cache'),
-            'current_instance_path': os.path.join(project_path, 'current_instance'),
-            'database_name': project_name,
+            'cache_path': os.path.join(vhost_path, 'cache'),
+            'current_instance_path': os.path.join(vhost_path, 'current_instance'),
+            'database_name': env.project_name,
             'environment': env.environment,
             'hosts': env.hosts,
-            'log_path': os.path.join(project_path, 'log'),
-            'media_path': os.path.join(project_path, 'media'),
-            'previous_instance_path': os.path.join(project_path, 'previous_instance'),
-            'project_root': env.projects_root,
-            'project_path': project_path,
-            'scripts_path': os.path.join(project_path, 'scripts'),
-            'user': project_name,
+            'log_path': os.path.join(vhost_path, 'log'),
+            'media_path': os.path.join(vhost_path, 'media'),
+            'previous_instance_path': os.path.join(vhost_path, 'previous_instance'),
+            'vhost_path': vhost_path,
+            'scripts_path': os.path.join(vhost_path, 'scripts'),
+            'user': '%s%s' % (env.project_name_prefix, env.project_name),
         })
 
 
@@ -68,7 +68,7 @@ class RemoteTask(Task):
     """
 
     requirements = [
-        'project_path',
+        'vhost_path',
         'project_name',
     ]
 
@@ -80,7 +80,7 @@ class RemoteTask(Task):
     def log(self, success):
         """ Single line task logging to ./log/fabric.log """
 
-        if success == True:
+        if success is True:
             result = 'success'
         else:
             result = 'failed'
@@ -119,13 +119,14 @@ class RemoteTask(Task):
             [require(r) for r in self.requirements]
 
             # update fabric environment for instance settings
-            instance_path = os.path.join(env.project_path, self.stamp)
+            instance_path = os.path.join(env.vhost_path, self.stamp)
             env.update({
                 'backup_path': os.path.join(instance_path, 'backup'),
                 'instance_stamp': self.stamp,
                 'instance_path': instance_path,
                 'source_path': os.path.join(instance_path, env.project_name),
-                'project_source_path': os.path.join(instance_path, env.project_name, getattr(env, 'project_source_folder', '')),
+                'project_path': os.path.join(instance_path, env.project_name),
+                'project_project_path': os.path.join(instance_path, env.project_name, env.project_name),
                 'virtualenv_path': os.path.join(instance_path, 'env'),
             })
 
@@ -140,13 +141,13 @@ class Deployment(RemoteTask):
         Usage:
 
         # default deployment using current git HEAD
-        $ fab staging.deploy
+        $ fab staging deploy
 
         # deployment for a specific git branch
-        $ fab staging.deploy:branch=my-branch
+        $ fab staging deploy:branch=my-branch
 
         # deployment for a specific git commit ID
-        $ fab staging.deploy:commit=1ec9d293ce54647df7f15ee7c0295b8eb2a5cbef
+        $ fab staging deploy:commit=1ec9d293ce54647df7f15ee7c0295b8eb2a5cbef
     """
 
     name = 'deploy'
@@ -163,11 +164,11 @@ class Deployment(RemoteTask):
         with settings(hide('warnings', 'running', 'stdout', 'stderr'), warn_only=True):
 
             # deploy by local commit
-            if kwargs.has_key('commit'):
+            if 'commit' in kwargs:
                 self.stamp = kwargs['commit']
 
             # deploy by HEAD for branch
-            elif kwargs.has_key('branch'):
+            elif 'branch' in kwargs:
                 self.stamp = utils.source.get_commit_id(kwargs['branch'])
 
             # deploy by local HEAD for local current branch
@@ -210,39 +211,80 @@ class Deployment(RemoteTask):
             for folder in folders_to_create:
                 utils.commands.create_folder(folder)
 
+            # before_deploy_source pause
+            if ('before_deploy_source' in pause_at):
+                print(green('\nOpening remote shell - before_deploy_source.'))
+                open_shell()
+
+            # before_deploy_source hook
+            if ('before_deploy_source' in env):
+                env.before_deploy_source(env, *args, **kwargs)
+
             print(green('\nDeploying source.'))
             utils.source.transfer_source(upload_path=env.source_path, tree=self.stamp)
 
+            # before_create_virtualenv pause
+            if ('before_create_virtualenv' in pause_at):
+                print(green('\nOpening remote shell - before_create_virtualenv.'))
+                open_shell()
+
+            # before_create_virtualenv hook
+            if ('before_create_virtualenv' in env):
+                env.before_create_virtualenv(env, *args, **kwargs)
+
             print(green('\nCreating virtual environment.'))
             utils.instance.create_virtualenv(env.virtualenv_path, env.user)
 
-            print(green('\nCopying .pth files.'))
-            put('%s/*.pth' % getattr(env, 'project_source_folder', '.'), '%s/lib/python2.6/site-packages' % env.virtualenv_path)
+            # before_pip_install pause
+            if ('before_pip_install' in pause_at):
+                print(green('\nOpening remote shell - before_pip_install.'))
+                open_shell()
+
+            # before_pip_install hook
+            if ('before_pip_install' in env):
+                env.before_pip_install(env, *args, **kwargs)
+
+            if exists(os.path.join(env.project_path, '*.pth')):
+                print(green('\nCopying .pth files.'))
+                utils.commands.copy(
+                    from_path=os.path.join(env.project_path, '*.pth'),
+                    to_path='%s/lib/python2.6/site-packages' % env.virtualenv_path
+                )
 
             print(green('\nPip installing requirements.'))
+            # TODO: use requirements_path instead of project_path?
             utils.instance.pip_install_requirements(
                 env.virtualenv_path,
-                env.project_source_path,
+                env.project_path,
                 env.cache_path,
                 env.log_path
             )
 
+            # after_pip_install pause
+            if ('after_pip_install' in pause_at):
+                print(green('\nOpening remote shell - after_pip_install.'))
+                open_shell()
+
+            # after_pip_install hook
+            if ('after_pip_install' in env):
+                env.after_pip_install(env, *args, **kwargs)
+
             print(green('\nCopying settings.py.'))
             utils.commands.copy(
-                from_path = os.path.join(env.project_path, 'settings.py'),
-                to_path = os.path.join(env.project_source_path, 'settings.py')
+                from_path=os.path.join(env.vhost_path, 'settings.py'),
+                to_path=os.path.join(env.project_project_path, 'settings.py')
             )
 
             print(green('\nLinking media folder.'))
             utils.commands.create_symbolic_link(
-                real_path = os.path.join(env.project_path, 'media'),
-                symbolic_path = os.path.join(env.project_source_path, 'media')
+                real_path=os.path.join(env.vhost_path, 'media'),
+                symbolic_path=os.path.join(env.project_path, 'media')
             )
 
             print(green('\nCollecting static files.'))
             utils.commands.django_manage(
                 env.virtualenv_path,
-                env.project_source_path,
+                env.project_path,
                 'collectstatic --link --noinput --verbosity=0 --traceback'
             )
         except:
@@ -263,20 +305,31 @@ class Deployment(RemoteTask):
             )
 
             with settings(show('stdout')):
+
+                # before_syncdb pause
                 if ('before_syncdb' in pause_at):
-                    print(green('\nOpening remote shell.'))
+                    print(green('\nOpening remote shell - before_syncdb.'))
                     open_shell()
 
+                # before_syncdb hook
+                if ('before_syncdb' in env):
+                    env.before_syncdb(env, *args, **kwargs)
+
                 print(green('\nSyncing database.'))
-                utils.commands.django_manage(env.virtualenv_path, env.project_source_path, 'syncdb')
+                utils.commands.django_manage(env.virtualenv_path, env.project_path, 'syncdb')
                 print('')
 
+                # before_migrate pause
                 if ('before_migrate' in pause_at):
-                    print(green('\nOpening remote shell.'))
+                    print(green('\nOpening remote shell - before_migrate.'))
                     open_shell()
 
+                # before_migrate hook
+                if ('before_migrate' in env):
+                    env.before_migrate(env, *args, **kwargs)
+
                 print(green('\nMigrating database.'))
-                utils.commands.django_manage(env.virtualenv_path, env.project_source_path, 'migrate')
+                utils.commands.django_manage(env.virtualenv_path, env.project_path, 'migrate')
                 print('')
 
             print(green('\nBacking up database at end.'))
@@ -300,34 +353,44 @@ class Deployment(RemoteTask):
 
             abort(red('Deploy failed and was rolled back.'))
 
+        # before_restart pause
         if ('before_restart' in pause_at):
-            print(green('\nOpening remote shell.'))
+            print(green('\nOpening remote shell - before_restart.'))
             open_shell()
 
+        # before_restart hook
+        if ('before_restart' in env):
+            env.before_restart(env, *args, **kwargs)
+
         print(green('\nUpdating instance symlinks.'))
-        utils.instance.set_current_instance(env.project_path, env.instance_path)
+        utils.instance.set_current_instance(env.vhost_path, env.instance_path)
 
         print(green('\nRestarting website.'))
-        utils.commands.touch_wsgi(env.project_path)
+        utils.commands.touch_wsgi(env.vhost_path)
 
+        # after_restart pause
         if ('after_restart' in pause_at):
-            print(green('\nOpening remote shell.'))
+            print(green('\nOpening remote shell - after_restart.'))
             open_shell()
 
+        # after_restart hook
+        if ('after_restart' in env):
+            env.after_restart(env, *args, **kwargs)
+
         self.log(success=True)
         self.prune_instances()
 
     def prune_instances(self):
         """ Find old instances and remove them to free up space """
 
-        old_instances = utils.instance.get_obsolete_instances(env.project_path)
+        old_instances = utils.instance.get_obsolete_instances(env.vhost_path)
 
         for instance in old_instances:
             is_current = bool(utils.instance.get_instance_stamp(env.current_instance_path) == instance)
             is_previous = bool(utils.instance.get_instance_stamp(env.previous_instance_path) == instance)
 
             if not (is_current or is_previous):
-                utils.commands.delete(os.path.join(env.project_path, instance))
+                utils.commands.delete(os.path.join(env.vhost_path, instance))
 
         if len(old_instances) > 0:
             print(green('\nThese old instances were removed from remote filesystem:'))
@@ -357,10 +420,10 @@ class Rollback(RemoteTask):
             )
 
             print(green('\nRemoving this instance and set previous to current.'))
-            utils.instance.rollback(env.project_path)
+            utils.instance.rollback(env.vhost_path)
 
             print(green('\nRestarting website.'))
-            utils.commands.touch_wsgi(env.project_path)
+            utils.commands.touch_wsgi(env.vhost_path)
 
             print(green('\nRemoving this instance from filesystem.'))
             utils.commands.delete(env.instance_path)
@@ -379,9 +442,6 @@ class Status(RemoteTask):
 
     def __call__(self, *args, **kwargs):
 
-        print(green('\nCurrent size of entire project:'))
-        print(utils.commands.get_folder_size(env.project_path))
-
         print(green('\nCurrent instance:'))
         current_instance = utils.commands.read_link(env.current_instance_path)
         if current_instance != env.current_instance_path:
@@ -403,6 +463,17 @@ class Status(RemoteTask):
             print(red('[empty]'))
 
 
+class Size(RemoteTask):
+    """ REMO - Show project size on remote host """
+
+    name = 'size'
+
+    def __call__(self, *args, **kwargs):
+
+        print(green('\nCurrent size of entire project:'))
+        print(utils.commands.get_folder_size(env.project_path))
+
+
 class Media(RemoteTask):
     """ REMO - Download media files (as archive) """
 
@@ -414,12 +485,12 @@ class Media(RemoteTask):
         cwd = os.getcwd()
 
         print(green('\nCompressing remote media folder.'))
-        utils.commands.create_tarball(env.project_path, 'media', file_name)
+        utils.commands.create_tarball(env.vhost_path, 'media', file_name)  # TODO: media_path?
 
         print(green('\nDownloading tarball.'))
         utils.commands.download_file(
-            remote_path = os.path.join(env.project_path, file_name),
-            local_path = os.path.join(cwd, file_name)
+            remote_path=os.path.join(env.vhost_path, file_name),
+            local_path=os.path.join(cwd, file_name)
         )
 
         print(green('\nSaved media tarball to:'))
@@ -446,8 +517,8 @@ class Database(RemoteTask):
 
         print(green('\nDownloading and removing remote backup.'))
         utils.commands.download_file(
-            remote_path = os.path.join(env.backup_path, file_name),
-            local_path = os.path.join(os.getcwd(), file_name)
+            remote_path=os.path.join(env.backup_path, file_name),
+            local_path=os.path.join(os.getcwd(), file_name)
         )
 
         print(green('\nSaved backup to:'))
